public class TwoSat { // for Kosaraju.scc(DirGraph) see previous page
    public static <T> boolean isSatisfiable(List<Clause<T>> formula) {
        Set<T> vars = new HashSet<T>();
        for (Clause<T> clause: formula) {
            vars.add(clause.first.value);
            vars.add(clause.second.value);
        }
        DirectedGraph<Literal<T>> impls = new DirectedGraph<Literal<T>>();
        for (T variable: vars) {
            impls.addNode(new Literal<T>(variable, true));
            impls.addNode(new Literal<T>(variable, false));
        }
        for (Clause<T> clause: formula) {
            impls.addEdge(clause.first.negation(), clause.second);
            impls.addEdge(clause.second.negation(), clause.first);
        }
        Map<Literal<T>, Integer> scc = Kosaraju.scc(impls);
        for (T variable: vars) {
            if (scc.get(new Literal<T>(variable, true))
                    .equals(scc.get(new Literal<T>(variable, false))))
                return false;
        }
        return true;
    }
} 
