public class ClosestPair {
    public static class Point {
        public final double x, y;
        public Point(double x, double y) { this.x = x; this.y = y; }
        public String toString() {  return "(" + x + ", " + y + ")";  }
    }
    public static class Pair {
        public Point point1 = null, point2 = null;
        public double distance = 0.0;
        public Pair() {  }
        public Pair(Point point1, Point point2) {
            this.point1 = point1; this.point2 = point2;
            this.distance = distance(point1, point2); 
        }
        public void update(Point point1, Point point2, double distance) {
            this.point1 = point1; this.point2 = point2;
            this.distance = distance;
        }
    }
    public static double distance(Point p1, Point p2) {
        return Math.hypot(p2.x - p1.x, p2.y - p1.y);
    }
    public static void sortByX(List<? extends Point> points) { 
        public int compare(Point point1, Point point2) {
            if (point1.x < point2.x) return -1;
            if (point1.x > point2.x) return 1; return 0;
        }
    }
    public static void sortByY(List<? extends Point> points) { 
        public int compare(Point point1, Point point2) {
            if (point1.y < point2.y) return -1;
            if (point1.y > point2.y) return 1; return 0;
        }
    }
    public static Pair bruteForce(List<? extends Point> points) {
        int numPoints = points.size();
        if (numPoints < 2) return null;
        Pair pair = new Pair(points.get(0), points.get(1));
        if (numPoints > 2) {
            for (int i = 0; i < numPoints - 1; i++) {
                Point point1 = points.get(i);
                for (int j = i + 1; j < numPoints; j++) {
                    Point point2 = points.get(j);
                    double distance = distance(point1, point2);
                    if (distance < pair.distance)
                        pair.update(point1, point2, distance);
                }
            }
        }
        return pair;
    }
    public static Pair divideAndConquer(List<Point> points) {
        List<Point> pSortX = new ArrayList<>(points); sortByX(pSortX);
        List<Point> pSortY = new ArrayList<>(points); sortByY(pSortY);
        return divideAndConquer(pSortX, pSortY);
    }
    static Pair divideAndConquer(List<Point> pSortX, List<Point> pSortY) {
        int numPoints = pSortX.size();
        if (numPoints <= 3) return bruteForce(pSortX);
        int divIndex = numPoints >>> 1;
        List<Point> leftOfCenter = pSortX.subList(0, divIndex);
        List<Point> rightOfCenter = pSortX.subList(divIndex, numPoints);
        List<Point> tempList = new ArrayList<>(leftOfCenter);
        sortByY(tempList);
        Pair closestPair = divideAndConquer(leftOfCenter, tempList);
        tempList.clear();
        tempList.addAll(rightOfCenter);
        sortByY(tempList);
        Pair closestPairRight = divideAndConquer(rightOfCenter, tempList);
        if (closestPairRight.distance < closestPair.distance)
            closestPair = closestPairRight;
        tempList.clear();
        double shortestDistance = closestPair.distance;
        double centerX = rightOfCenter.get(0).x;
        for (Point point : pSortY)
            if (Math.abs(centerX - point.x) < shortestDistance)
                tempList.add(point);
        for (int i = 0; i < tempList.size() - 1; i++) {
            Point point1 = tempList.get(i);
            for (int j = i + 1; j < tempList.size(); j++) {
                Point point2 = tempList.get(j);
                if ((point2.y - point1.y) >= shortestDistance) break;
                double distance = distance(point1, point2);
                if (distance < closestPair.distance) {
                    closestPair.update(point1, point2, distance);
                    shortestDistance = distance;
                }
            }
        }
        return closestPair;
    }
}
