class ConvexHull {
    static final double EPSILON = 0.0001;
    public static Point[] getConvexHull(Point[] p) {
        Arrays.sort(p);
        List<Point> hull = new ArrayList<Point>();
        hull.add(p[0]);
        for (int i = 1, min = hull.size() + 1; i < p.length; i++) {
            hull.add(p[i]);
            while (hull.size() > min && !endsWithRightTurn(hull))
                hull.remove(hull.size() - 2);
        }
        for (int i = p.length - 2, min = hull.size() + 1; i >= 0; i--) {
            hull.add(p[i]);
            while (hull.size() > min && !endsWithRightTurn(hull))
                hull.remove(hull.size() - 2);
        }
        hull.remove(hull.size() - 1);
        return hull.toArray(new Point[hull.size()]);
    }

    private static boolean endsWithRightTurn(List<Point> hull) {
        Point a = hull.get(hull.size() - 3);
        Point b = hull.get(hull.size() - 2);
        Point c = hull.get(hull.size() - 1);
        return area(a, b, c) < -EPSILON;
    }

    private static double area(Point a, Point b, Point c) {
        return (a.x*b.y-a.y*b.x+a.y*c.x-a.x*c.y+b.x*c.y-c.x*b.y)/2;
    }

    public static class Point implements Comparable<Point> {
        public double x, y;
        public Point(double x, double y) { this.x = x; this.y = y; }

        @Override
        public int compareTo(Point that) {
            if (this.x < that.x) return -1;
            if (this.x > that.x) return 1;
            if (this.y < that.y) return -1;
            if (this.y > that.y) return 1;
            return 0;
        }
    }
}
