public final class Kosaraju { // for DirGraph see 2-SAT
    public static <T> Map<T, Integer> scc(DirGraph<T> g) {
        Stack<T> visitOrder = dfsVisitOrder(graphReverse(g));
        Map<T, Integer> result = new HashMap<T, Integer>();
        int iteration = 0;
        while (!visitOrder.isEmpty()) {
            T startPoint = visitOrder.pop();
            if (result.containsKey(startPoint)) continue;
            markRNodes(startPoint, g, result, iteration);
            ++iteration;
        }
        return result;
    }
    static <T> DirGraph<T> graphReverse(DirGraph<T> g) {
        DirGraph<T> result = new DirGraph<T>();
        for (T node: g) result.addNode(node);
        for (T node: g) { for (T endpoint: g.edgesFrom(node)) {
                result.addEdge(endpoint, node); }}
        return result;
    }
    static <T> Stack<T> dfsVisitOrder(DirGraph<T> g) {
        Stack<T> result = new Stack<T>();
        Set<T> visited = new HashSet<T>();
        for (T node: g) recExplore(node, g, result, visited);
        return result;
    }
    static <T> void recExplore(T node, DirGraph<T> g,
            Stack<T> result, Set<T> visited) {
        if (visited.contains(node)) return;
        visited.add(node);
        for (T endpoint: g.edgesFrom(node))
            recExplore(endpoint, g, result, visited);
        result.push(node);
    }
    static <T> void markRNodes(T node,DirGraph<T> g,Map<T,Int> r,int l) {
        if (r.containsKey(node)) return; r.put(node, l);
        for (T endpoint: g.edgesFrom(node)) markRNodes(endpoint, g, r, l);
    }
} 
